#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "itemscrollbox.h"

#include "itemsettingsdialog.h"

MainWindow::MainWindow(Microcontroller *micro, PowerItem* powerItem, ItemStore* itemStore, SensorStore *sensorStore, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    colorChooser(this),
    _micro(micro),
    _powerItem(powerItem)
{
    ui->setupUi(this);

    connect(ui->pushButton_power, SIGNAL(clicked()), this, SLOT(showPowerItemDialog()));

    //Relays
    connect(ui->pushButton_refesh, SIGNAL(clicked()), _micro, SLOT(requestState()));
    connect(itemStore, &ItemStore::itemAdded, ui->relayList, &ItemScrollBox::addItem);
    connect(itemStore, &ItemStore::itemDeleted, ui->relayList, &ItemScrollBox::removeItem);

    //Sensors

    ui->sensorListView->sensorsChanged(*(sensorStore->getSensors()));
    connect(sensorStore, &SensorStore::stateChenged, ui->sensorListView, &SensorListWidget::sensorsChanged);

    //RGB Leds
    connect(&colorChooser, SIGNAL(colorSelected(const QColor)), this, SLOT(slotChangedRgb(const QColor)));
    connect(ui->button_quit, SIGNAL(clicked()), this, SLOT(close()));
    connect(ui->button_color, SIGNAL(clicked()), &colorChooser, SLOT(show()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::showPowerItemDialog()
{
    ItemSettingsDialog diag(_powerItem, this);
    diag.show();
    diag.exec();
}

void MainWindow::slotChangedRgb(const QColor color)
{
    _micro->changeRgbColor(color);
}

void MainWindow::changeHeaderLableText(QString string)
{
    if(string.size() > 28)
    {
        string.remove(28, string.size()-(28));
        string.append("...");
    }
    ui->label_serialRecive->setText(string);
}
